@@ -176,6 +176,9 @@ bool CB_window_close(gMainWindow *sender)
 	if (!THIS)
 		return false;
 
+	if (gMainWindow::_current && !sender->isCurrentLoopLevel())
+		return true;
+	
 	if (GB.Raise(THIS, EVENT_Close, 0))
 		return true;
 
@@ -633,7 +636,7 @@ END_PROPERTY
 BEGIN_PROPERTY(CWINDOW_stacking)
 
 	if (READ_PROPERTY)
-		GB.ReturnInteger(WINDOW->getStacking());
+		GB.ReturnInteger(WINDOW->stacking());
 	else
 		WINDOW->setStacking(VPROP(GB_INTEGER));
 
