@@ -83,9 +83,6 @@ static gControl *find_child(gControl *control, int rx, int ry, gControl *button_
 	gMainWindow *window;
 	int x, y;
 	int cx, cy, cw, ch;
-	#ifdef GTK3
-	GtkAllocation a;
-	#endif
 
 	if (gApplication::_control_grab)
 		return gApplication::_control_grab;
@@ -104,13 +101,6 @@ static gControl *find_child(gControl *control, int rx, int ry, gControl *button_
 	if (skip_ignore_mouse && control->isIgnoreMouse())
 		return NULL;
 	
-	#ifdef GTK3
-	gtk_widget_get_allocation(window->frame, &a);
-	//fprintf(stderr, "find_child: %d %d window: %d %d %d %d\n", rx, ry, a.x, a.y, a.width, a.height);
-	rx -= a.x;
-	ry -= a.y;
-	#endif
-
 	#if DEBUG_FIND_CONTROL
 	fprintf(stderr, "find_child: [%s %p] %s (%d %d)\n", window->name(), window, control->name(), rx, ry);
 	#endif
@@ -118,22 +108,15 @@ static gControl *find_child(gControl *control, int rx, int ry, gControl *button_
 	while (control->isContainer())
 	{
 		control->getScreenPos(&x, &y);
-		#ifdef GTK3
-		if (!control->isTopLevel())
-		{
-			x -= a.x;
-			y -= a.y;
-		}
-		#endif
 
 		#if DEBUG_FIND_CONTROL
-		fprintf(stderr, "  screen pos %s = %d %d\n", control->name(), x ,y);
+		fprintf(stderr, "  screen pos of %s: %d %d\n", control->name(), x ,y);
 		#endif
 		
 		cont = (gContainer *)control;
 
-		cx = cont->clientX();
-		cy = cont->clientY();
+		cx = x + cont->clientX();
+		cy = y + cont->clientY();
 		cw = cont->clientWidth();
 		ch = cont->clientHeight();
 
@@ -141,9 +124,7 @@ static gControl *find_child(gControl *control, int rx, int ry, gControl *button_
 		fprintf(stderr, "  client area of %s: %d %d %d %d\n", control->name(), cx, cy, cw, ch);
 		#endif
 
-		x = rx - x;
-		y = ry - y;
-		if (x < cx || y < cy || x >= (cx + cw) || y >= (cy + ch))
+		if (rx < cx || ry < cy || rx >= (cx + cw) || ry >= (cy + ch))
 		{
 			#if DEBUG_FIND_CONTROL
 			fprintf(stderr, "  outside of client area of %s\n", control->name());
@@ -153,9 +134,9 @@ static gControl *find_child(gControl *control, int rx, int ry, gControl *button_
 		}
 
 		#if DEBUG_FIND_CONTROL
-		fprintf(stderr, "  find coord %d %d\n", x, y);
+		fprintf(stderr, "  find child at %d %d / %d %d\n", rx, ry, rx - cx, ry - cy);
 		#endif
-		child = cont->find(x, y, skip_ignore_mouse);
+		child = cont->find(rx - cx, ry - cy, skip_ignore_mouse);
 		if (!child)
 			break;
 
