@@ -568,27 +568,24 @@ POSITION AND SIZE
 
 ******************************************************************/
 
-bool gControl::getScreenPos(int *x, int *y)
+void gControl::getScreenPos(int *x, int *y)
 {
-	if (!gtk_widget_get_window(border))
+	if (gtk_widget_get_has_window(border))
 	{
-		if (pr)
-		{
-			pr->getScreenPos(x, y);
-			x += pr->clientX() + bufX;
-			y += pr->clientY() + bufY;
-			return false;
-		}
-		
-		*x = *y = 0; // widget is not realized
-		return true;
+		GdkWindow *win = gtk_widget_get_window(border);
+		if (win)
+			gdk_window_get_origin(win, x, y);
+		else
+			*x = *y = 0;
+	}
+	else
+	{
+		pr->getScreenPos(x, y);
+		*x += pr->clientX() + bufX;
+		*y += pr->clientY() + bufY;
 	}
 
-	gdk_window_get_origin(gtk_widget_get_window(border), x, y);
-
-	//fprintf(stderr, "getScreenPos: %s: %d %d: %d\n", name(), *x, *y, gtk_widget_get_has_window(border));
-
-	#if GTK_CHECK_VERSION(2, 18, 0)
+	/*#if GTK_CHECK_VERSION(2, 18, 0)
 	if (!gtk_widget_get_has_window(border))
 	{
 		GtkAllocation a;
@@ -596,10 +593,9 @@ bool gControl::getScreenPos(int *x, int *y)
 		*x += a.x;
 		*y += a.y;
 	}
-	#endif
+	#endif*/
 
 	//fprintf(stderr, "getScreenPos: --> %d %d\n", *x, *y);
-	return false;
 }
 
 int gControl::screenX()
@@ -1443,17 +1439,16 @@ static GList **get_children_list(GtkContainer *parent)
 #endif
 }
 
-void gControl::restack(bool raise)
+void gControl::raiseInternal()
 {
 	GtkContainer *parent;
 	GList **children;
 	GList *find;
-	gpointer *p;
 	GdkWindow *window;
 
-	if (!pr) 
+	if (pr->child(pr->childCount() - 1) == this)
 		return;
-
+	
 	parent = GTK_CONTAINER(gtk_widget_get_parent(border));
 	
 	//fprintf(stderr, "%s: %s -> %s (%s%s)\n", raise ? "raise" : "lower", name(), pr->name(), GTK_IS_LAYOUT(parent) ? "L" : "F", gtk_widget_get_has_window(border) ? "W" : "");
@@ -1471,40 +1466,57 @@ void gControl::restack(bool raise)
 		hideButKeepFocus();
 	
 	*children = g_list_remove_link(*children, find);
-	if (raise)
-		*children = g_list_concat(*children, find);
-	else
-		*children = g_list_concat(find, *children);
+	*children = g_list_concat(*children, find);
 	
 	if (gtk_widget_get_has_window(border))
 	{
 		window = gtk_widget_get_window(border);
 		if (window)
-		{
-			if (raise)
-				gdk_window_raise(window);
-			else
-				gdk_window_lower(window);
-		}
+			gdk_window_raise(window);
 	}
 	
 	g_ptr_array_remove(pr->_children, this);
-	
-	if (raise)
-	{
-		g_ptr_array_add(pr->_children, this);
-	}
-	else
-	{
-		g_ptr_array_add(pr->_children, NULL);
-		p = pr->_children->pdata;
-		memmove(&p[1], &p[0], (pr->_children->len - 1) * sizeof(gpointer));
-		p[0] = this;
-	}
+	g_ptr_array_add(pr->_children, this);
 	
 	if (_visible)
 		showButKeepFocus();
+}
 
+void gControl::restack(bool raise)
+{
+	int i;
+	gControl *child;
+	GPtrArray *copy;
+
+	if (!pr) 
+		return;
+
+	if (!raise)
+	{
+		if (pr->child(0) == this)
+			return;
+		
+		copy = g_ptr_array_new();
+		for (i = 0; i < pr->childCount(); i++)
+		{
+			child = pr->child(i);
+			if (child != this)
+				g_ptr_array_add(copy, child);
+		}
+		
+		for (i = 0; i < (int)copy->len; i++)
+		{
+			child = (gControl *)g_ptr_array_index(copy, i);
+			child->raiseInternal();
+		}
+		
+		g_ptr_array_unref(copy);
+	}
+	else
+	{
+		raiseInternal();
+	}
+	
 	updateGeometry(true);
 	pr->performArrange();
 	pr->refresh();
