@@ -38,6 +38,10 @@
 
 #include "CKey.h"
 
+#ifdef GTK3
+static bool _no_style_change = FALSE;
+#endif
+
 void stub(const char *function)
 {
 	printf("gb.gtk: warning: %s not yet implemented\n", function);
@@ -2144,6 +2148,8 @@ static int type_to_index(GType type)
 		return 10;
 	else if (type == GTK_TYPE_WINDOW)
 		return 11;
+	else if (type == GTK_TYPE_LINK_BUTTON)
+		return 12;
 	else
 		return 0;
 }
@@ -2155,7 +2161,8 @@ const char *gt_get_style_class(GType type)
 	static const char *_class[] = {
 		GTK_STYLE_CLASS_DEFAULT, GTK_STYLE_CLASS_ENTRY, GTK_STYLE_CLASS_BACKGROUND, GTK_STYLE_CLASS_TOOLTIP,
 		GTK_STYLE_CLASS_SCROLLBAR, GTK_STYLE_CLASS_DEFAULT, GTK_STYLE_CLASS_CHECK, GTK_STYLE_CLASS_RADIO,
-		GTK_STYLE_CLASS_FRAME, GTK_STYLE_CLASS_BACKGROUND, GTK_STYLE_CLASS_BUTTON, GTK_STYLE_CLASS_BACKGROUND
+		GTK_STYLE_CLASS_FRAME, GTK_STYLE_CLASS_BACKGROUND, GTK_STYLE_CLASS_BUTTON, GTK_STYLE_CLASS_BACKGROUND,
+		GTK_STYLE_CLASS_BUTTON
 	};
 
 	int index = type_to_index(type);
@@ -2165,26 +2172,30 @@ const char *gt_get_style_class(GType type)
 		return _class[index];
 }
 
-static void cb_style_changed(GtkStyleContext *style, gpointer)
+static void cb_style_changed(GtkStyleContext *style, gpointer data)
 {
+	if (_no_style_change)
+		return;
 	gDesktop::onStyleChange();
 }
 
-GtkStyleContext *gt_get_style(GType type, const char *node, const char *more_klass)
+GtkStyleContext *gt_get_style(GType type, const char *node, const char *more_klass, GtkStateFlags state)
 {
 	int index = 0;
 	GtkStyleContext *style;
+	GtkWidgetPath *path;
+	const char *klass;
 
 	if (!node && !more_klass)
 	{
 		index = type_to_index(type);
 		style = _style[index];
 		if (style)
-			return style;
+			goto RETURN_CACHED_STYLE;
 	}
 	
-	GtkWidgetPath *path = gtk_widget_path_new();
-	const char *klass = gt_get_style_class(type);
+	path = gtk_widget_path_new();
+	klass = gt_get_style_class(type);
 
 	style = gtk_style_context_new();
 
@@ -2198,7 +2209,7 @@ GtkStyleContext *gt_get_style(GType type, const char *node, const char *more_kla
 
 	#if GTK_CHECK_VERSION(3, 20, 0)
 	gtk_widget_path_iter_set_object_name(path, -1, klass);
-	if (node)
+	if (node && *node)
 	{
 		gtk_widget_path_append_type(path, type);
 		gtk_widget_path_iter_set_object_name(path, 1, node);
@@ -2206,11 +2217,23 @@ GtkStyleContext *gt_get_style(GType type, const char *node, const char *more_kla
 	#endif
 
 	gtk_style_context_set_path(style, path);
+	gtk_widget_path_free(path);
 
-	if (!node && !more_klass)
-		_style[index] = style;
+	if (node || more_klass)
+		return style;
+	
+	_style[index] = style;
+	g_signal_connect(G_OBJECT(style), "changed", G_CALLBACK(cb_style_changed), (gpointer)(intptr_t)index);
+	
+RETURN_CACHED_STYLE:
+	
+	if (state)
+	{
+		_no_style_change = TRUE;
+		gtk_style_context_set_state(style, state);
+		_no_style_change = FALSE;
+	}
 	
-	g_signal_connect(G_OBJECT(style), "changed", G_CALLBACK(cb_style_changed), 0);
 	return style;
 }
 
@@ -2293,15 +2316,13 @@ void gt_draw_border(cairo_t *cr, GtkStyleContext *st, GtkStateFlags state, int b
 
 	if (border == BORDER_RAISED)
 	{
-		st = gt_get_style(GTK_TYPE_BUTTON);
-		gtk_style_context_set_state(st, state);
+		st = gt_get_style(GTK_TYPE_BUTTON, NULL, NULL, state);
 		//gtk_style_context_add_class(st, GTK_STYLE_CLASS_BUTTON);
 		gtk_render_frame(st, cr, x, y, w, h);
 	}
 	else if (border == BORDER_SUNKEN)
 	{
-		st = gt_get_style(GTK_TYPE_ENTRY);
-		gtk_style_context_set_state(st, state);
+		st = gt_get_style(GTK_TYPE_ENTRY, NULL, NULL, state);
 		//gtk_style_context_add_class(st, GTK_STYLE_CLASS_ENTRY);
 		if (bg)
 			gtk_render_background(st, cr, x, y, w, h);
@@ -2309,8 +2330,7 @@ void gt_draw_border(cairo_t *cr, GtkStyleContext *st, GtkStateFlags state, int b
 	}
 	else if (border == BORDER_ETCHED)
 	{
-		st = gt_get_style(GTK_TYPE_FRAME);
-		gtk_style_context_set_state(st, state);
+		st = gt_get_style(GTK_TYPE_FRAME, NULL, NULL, state);
 		//gtk_style_context_add_class(st, GTK_STYLE_CLASS_FRAME);
 		gtk_render_frame(st, cr, x, y, w, h);
 	}
@@ -2696,6 +2716,5 @@ void gt_layout_get_extents(PangoLayout *layout, int *w, int *h, bool pixels)
 		*w = gt_pango_to_pixel(*w); 
 		*h = gt_pango_to_pixel(*h); 
 	}
-	
 }
 
