@@ -652,9 +652,10 @@ BEGIN_METHOD(File_SetBaseName, GB_STRING path; GB_STRING new_basename)
 
 END_METHOD
 
-static void error_CFILE_load_save(STREAM *stream)
+static void error_CFILE_load_save(STREAM *stream, char **str)
 {
 	STREAM_close(stream);
+	STRING_free(str);
 }
 
 BEGIN_METHOD(File_Load, GB_STRING path)
@@ -666,7 +667,7 @@ BEGIN_METHOD(File_Load, GB_STRING path)
 
 	STREAM_open(&stream, STRING_conv_file_name(STRING(path), LENGTH(path)), GB_ST_READ);
 	
-	ON_ERROR_1(error_CFILE_load_save, &stream)
+	ON_ERROR_2(error_CFILE_load_save, &stream, &str)
 	{
 		STREAM_lof(&stream, &len);
 		if (len >> 31)
