@@ -734,7 +734,7 @@ double DATE_to_double(struct timeval *time, int from_start)
 }
 
 
-bool DATE_timer(double *result, int from_start)
+bool DATE_timer(double *result, bool from_start)
 {
 	*result = get_monotonic_timer();
 	
